/*
 * MEUCommandLine.java
 * The visual command line.
 * Created on July 31, 2000, 11:35 PM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

/** The command line at the top of the MEU window.
 * @author jax
 * @version 1.0
 */
public class MEUCommandLine extends javax.swing.JPanel {

  /** Associated frame controller */
  private MEUController controller;

  /** Custom keymap  */
  private javax.swing.text.Keymap keymap;

  /** Creates new form MEUCommandLine */
  public MEUCommandLine() {
    initComponents ();
    addCustomKeymap();
  }

  /** Creates new form MEUCommandLine and assigns controller.
   * @param c The controller.
   */
  public MEUCommandLine(MEUController c) {
    setController(c);
    initComponents ();
    addCustomKeymap();
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    commandLine = new javax.swing.JTextField ();
    clearButton = new javax.swing.JButton ();
    setLayout (new java.awt.BorderLayout ());

    commandLine.setToolTipText ("Enter an editing command here and press return");
    commandLine.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        commandLineActionPerformed (evt);
      }
    }
    );


    add (commandLine, java.awt.BorderLayout.CENTER);

    clearButton.setToolTipText ("Press here to clear the command line");
    clearButton.setHorizontalTextPosition (javax.swing.SwingConstants.CENTER);
    clearButton.setText (" MEU ");
    clearButton.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        clearButtonActionPerformed (evt);
      }
    }
    );


    add (clearButton, java.awt.BorderLayout.WEST);

  }//GEN-END:initComponents

  private void clearButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearButtonActionPerformed
    // Add your handling code here:
    commandLine.setText("");
    commandLine.requestFocus();
  }//GEN-LAST:event_clearButtonActionPerformed

  private void commandLineActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_commandLineActionPerformed
    // Add your handling code here:
    // Just for design until we add command functionality
    if (null != controller) {
      controller.doCommand(commandLine.getText());
    }
    else {
      System.err.println("Command line found no controller set.");
    }
  }//GEN-LAST:event_commandLineActionPerformed


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JTextField commandLine;
  private javax.swing.JButton clearButton;
  // End of variables declaration//GEN-END:variables

  /* Set controller (at ctor time) */
  /** Set the frame controller.
   * @param c The frame controller.
   */
  protected void setController(MEUController c) {
    controller = c;
  }

  private void addCustomKeymap() {
    keymap = commandLine.addKeymap("com.softwoehr.jtopencontrib.meu.keymap.commandline", commandLine.getKeymap());

    keymap.addActionForKeyStroke(javax.swing.KeyStroke.getKeyStroke
    (java.awt.event.KeyEvent.VK_F3, 0) ,
    new javax.swing.AbstractAction () {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        controller.actionClose(e);
      }
    });

    keymap.addActionForKeyStroke(javax.swing.KeyStroke.getKeyStroke
    (java.awt.event.KeyEvent.VK_F4, 0) ,
    new javax.swing.AbstractAction () {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        controller.actionSave(e);
      }
    });

    keymap.addActionForKeyStroke(javax.swing.KeyStroke.getKeyStroke
    (java.awt.event.KeyEvent.VK_F10, 0) ,
    new javax.swing.AbstractAction () {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        controller.actionToggleFocus(e);
      }
    });

    commandLine.setKeymap(keymap);
  }

  /** Set the focus to the command line.
   * The MEUFrame calls this is toggleFocus().
   */
  public void commandLineFocus() {
    commandLine.requestFocus();
  }
}
