/*
 * MEUFileMember.java
 * This is the actual records for reading and writing on the server.
 * Created on July 30, 2000, 2:28 AM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

//import java.io.*;

/** Represents the AS400 file member which is being edited.
 * @author jax
 * @version 1.0
 */
public class MEUFileMember extends Object {

  /** Record field */
  public static final int SRCSEQ_INDEX = 0;
  /** Record field */
  public static final int SRCDAT_INDEX = 1;
  /** Record field */
  public static final int SRCDTA_INDEX = 2;

  /** Used in resequencing */
  public static final java.math.BigDecimal ONE_POINT_OH_OH = new java.math.BigDecimal("1.00");

  private com.ibm.as400.access.AS400 system;
  private com.ibm.as400.access.SequentialFile as400File;
  private com.ibm.as400.access.AS400FileRecordDescription as400FileRecordDescription;
  private com.ibm.as400.access.RecordFormat recordFormats [];
  private com.ibm.as400.access.Record records [];
  private java.util.Vector workingRecords;
  private com.ibm.as400.access.QSYSObjectPathName objectPath;
  private boolean isDirty = false;

  /** Creates new MEUFileMember */
  public MEUFileMember(com.ibm.as400.access.AS400 as400, String lib, String file, String member) {
    system = as400;
    instanceObjectPath(lib.toUpperCase(), file.toUpperCase(), member.toUpperCase());
    instanceFile();
    instanceFileRecordDescription();
    dfltWorkingRecords ();
  }

  /** Compose an IFS path from libname, filename, membername */
  private void instanceObjectPath(String lib, String file, String member) {
    objectPath = new com.ibm.as400.access.QSYSObjectPathName(lib, file, member, "MBR");
  }

  /** Instance the AS400File from member data of this instance. */
  private void instanceFile() {
    as400File = new com.ibm.as400.access.SequentialFile(system, getIFSPath());
  }

  /** Instance the record description getter from data of this instance. */
  private void instanceFileRecordDescription () {
    as400FileRecordDescription = new com.ibm.as400.access.AS400FileRecordDescription(system, getIFSPath());
  }

  /** Vector is always there so we can start with new file.
   * Is discarded in favor of a new vector if readAll() is performed.
   */
  private void dfltWorkingRecords () {
    workingRecords = new java.util.Vector(24,24);
  }

  /*****************/
  /* Member access */
  /*****************/

  /** Return server file resides on. */
  public com.ibm.as400.access.AS400 getSystem() {
    return system;
  }

  /** Return the object path object which holds the name of the file member */
  public com.ibm.as400.access.QSYSObjectPathName getObjectPath () {
    return objectPath;
  }

  /** Return the IFS path for the file member represented by this instance. */
  public String getIFSPath () {
    return objectPath.getPath();
  }

  /** Return the name of the which this MEUFileMember represents */
  public String getLibraryName() {
    return objectPath.getLibraryName();
  }

  /** Return the name of the object (file) which this MEUFileMember represents */
  public String getObjectName() {
    return objectPath.getObjectName();
  }

  /** Return the name of the member which this MEUFileMember represents */
  public String getMemberName() {
    return objectPath.getMemberName();
  }

  /*****************/
  /* Record access */
  /*****************/

  /** Get array of formats ... this is source file, will only have one format. */
  private void getRecordFormats()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException
  {
    recordFormats = as400FileRecordDescription.retrieveRecordFormat();
  }

  /** Get the record format for this source member. */
  public com.ibm.as400.access.RecordFormat getRecordFormatTheHardWay()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException
  {
    com.ibm.as400.access.RecordFormat result = null;
    if (null == recordFormats) {
      getRecordFormats();
    }
    result = recordFormats[0];
    return result;
  }

  // Get the record format
  /**
   * @throws AS400Exception
   * @throws AS400SecurityException
   * @throws InterruptedException
   * @throws IOException
   * @throws PropertyVetoException
   * @return
   */
  public com.ibm.as400.access.RecordFormat getRecordFormat()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException {
    // Also, if it's open, record format not null.
    if (null == as400File.getRecordFormat()) {
      as400File.setRecordFormat();
    } // If it's not null, we don't break the rule by doing setRecordFormat on an open file.
    return as400File.getRecordFormat();
  }

  /** Cause fetch of record format */
  public boolean setRecordFormat() {
    boolean result = false;
    try {
      result = (null != getRecordFormat());
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace();
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace();
    }
    catch (InterruptedException e) {
      e.printStackTrace();
    }
    catch (java.io.IOException e) {
      e.printStackTrace();
    }
    catch (java.beans.PropertyVetoException e) {
      e.printStackTrace();
    }
    return result;
  }

  /** Set record format if not open, also clearing records.
   * Return false if open.
   */
  public boolean setRecordFormat (com.ibm.as400.access.RecordFormat rf)
  throws java.beans.PropertyVetoException
  {
    boolean result = true;
    if (!isOpen()) {
      as400File.setRecordFormat(rf);
      removeAllRecords();
    }
    else {
      result = false;
    }
    return result;
  }

  /** Return the length of the source text field
   * @return the length of the field or zero (0) if can't get record format
   */
  public int getSRCDTAlength() {
    int result = 0;
    try {
      result = getRecordFormat().getFieldDescription(SRCDTA_INDEX).getLength();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }

  /** Get the text field by row number */
  public String getSRCDTAcontents (int recnum) {
    String result = "";
    if (recnum >=0 && recnum < getNumberOfRecords()) {
      try
      {
        result = getSRCDTA(getRecordAt(recnum));
      }
      catch (java.io.UnsupportedEncodingException e) {
        e.printStackTrace();
      }
    }
    return result;
  }

  /** Return a new record in this file's format. */
  public com.ibm.as400.access.Record getNewRecord()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException
  {
    return getRecordFormat().getNewRecord();
  }

  /** Get a new record with its fields filled in */
  public com.ibm.as400.access.Record getNewRecord(java.math.BigDecimal srcseq, java.math.BigDecimal srcdat, String srcdta)
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException
  {
    com.ibm.as400.access.Record r = getNewRecord();
    setSRCSEQ(r, srcseq);
    setSRCDAT(r, srcdat);
    setSRCDTA(r, srcdta);
    return r;
  }

  /** Set the sequence field */
  public void setSRCSEQ (com.ibm.as400.access.Record r, java.math.BigDecimal datum) {
    r.setField(SRCSEQ_INDEX, datum);
  }

  /** Set the date field */
  public void setSRCDAT (com.ibm.as400.access.Record r, java.math.BigDecimal datum) {
    r.setField(SRCDAT_INDEX, datum);
  }

  /** Set the text field */
  public void setSRCDTA (com.ibm.as400.access.Record r, String datum) {
    StringBuffer sb = new StringBuffer(getSRCDTAlength());
    for (int i = 0; i < getSRCDTAlength() ; i++) {
      sb.append(' ');
    }
    sb.replace(0, java.lang.Math.min(datum.length(), sb.length()), datum);
    r.setField(SRCDTA_INDEX, sb.toString());
  }

  /** Get the sequence field */
  public static java.math.BigDecimal getSRCSEQ (com.ibm.as400.access.Record r)
  throws  java.io.UnsupportedEncodingException
  {
    return (java.math.BigDecimal) r.getField(SRCSEQ_INDEX);
  }

  /** Get the date field */
  public static java.math.BigDecimal getSRCDAT (com.ibm.as400.access.Record r)
  throws  java.io.UnsupportedEncodingException
  {
    return (java.math.BigDecimal) r.getField(SRCDAT_INDEX);
  }

  /** Get the text field */
  public static String getSRCDTA (com.ibm.as400.access.Record r)
  throws  java.io.UnsupportedEncodingException
  {
    return (String) r.getField(SRCDTA_INDEX);
  }

  /******************************************************/
  /* Our collection of records which is the file member */
  /******************************************************/

  /** Insert a record into the working vector of records */
  public void insertRecord (com.ibm.as400.access.Record r, int index)
  throws ArrayIndexOutOfBoundsException
  {
    /* From java.util.Vector.insertElementAt() javadocs:
     * The index must be a value greater than or equal to 0 and less than or equal to the current size of the vector. (If the index is
     * equal to the current size of the vector, the new element is appended to the Vector.)
     */
    workingRecords.insertElementAt(r, index);
  }

  /** Append a record to the working vector of records */
  public void appendRecord (com.ibm.as400.access.Record r)
  {
    workingRecords.addElement(r);
  }

  /** Remove a record from our working vector of records */
  public void removeRecord (int index)
  throws ArrayIndexOutOfBoundsException
  {
    workingRecords.removeElementAt (index);
  }

  /** Remove all records from our working vector of records */
  public void removeAllRecords () {
    workingRecords.removeAllElements();
  }

  /** How many records are there? */
  public int getNumberOfRecords() {
    return workingRecords.size();
  }

  /** Return all records in an enumeration */
  public java.util.Enumeration elements() {
    return workingRecords.elements();
  }

  /** Get a specific record (from 0 to numrecs - 1). */
  public com.ibm.as400.access.Record getRecordAt(int index) {
    com.ibm.as400.access.Record result = null;
    if (0 <= index & index < getNumberOfRecords()) {
      result = (com.ibm.as400.access.Record) workingRecords.elementAt(index);
    }
    return result;
  }

  /** Reset the sequence numbers of all records */
  public void resequence() {
    java.math.BigDecimal seqnum   = new java.math.BigDecimal("1.00");
    for (java.util.Enumeration e = workingRecords.elements(); e.hasMoreElements(); ) {
      setSRCSEQ((com.ibm.as400.access.Record) e.nextElement(), seqnum);
      seqnum.add(ONE_POINT_OH_OH);
    }
  }

  /** Resequence a range of records.
   * It's the callers responsibility to determine that one doesn't overrun a later range of sequence numbers.
   */
  public void resequenceFrom(int index, int postindex, java.math.BigDecimal seqnum, java.math.BigDecimal increment) {
    index     = java.lang.Math.max(0, java.lang.Math.min(index, getNumberOfRecords()));
    postindex = java.lang.Math.min(java.lang.Math.max(index,postindex), getNumberOfRecords());
    for (; index < postindex; index++) {
      setSRCSEQ((com.ibm.as400.access.Record) workingRecords.elementAt(index), seqnum);
      seqnum = seqnum.add(increment);
    }
  }

  /***********************/
  /* File member control */
  /***********************/

  /** Return true if the system object referred to by the ifsPath exists */
  public static boolean pathExists (com.ibm.as400.access.AS400 sys, com.ibm.as400.access.QSYSObjectPathName qPath)
  throws java.io.IOException
  {
    com.ibm.as400.access.IFSFile ifsFile = new com.ibm.as400.access.IFSFile(sys, qPath.getPath());
    return ifsFile.exists();
  }

  /** Run this on an un-open'ed MEUFileMember that doesn't
   * yet exist on the server to create this member.
   * The member that is created is the one named by the MEUFileMember object.
   * This routine meant to be used to start edit session.
   */
  public void createMember (com.ibm.as400.access.RecordFormat rf, String textDescription)
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException
  {
    com.ibm.as400.access.QSYSObjectPathName myObjectPath =
    new com.ibm.as400.access.QSYSObjectPathName(getLibraryName(), getObjectName(), "FILE");
    com.ibm.as400.access.SequentialFile myFile = new com.ibm.as400.access.SequentialFile(system, myObjectPath.getPath());
    myFile.setRecordFormat(rf);
    myFile.addPhysicalFileMember(objectPath.getMemberName(),textDescription);
    // myAS400File.finalize() (to close any residual connection) ?
  }

  /** Run this on an un-open'ed MEUFileMember that doesn't
   * yet exist on the server to create this member.
   * The member that is created is the one named by the MEUFileMember object.
   * This routine meant to be used to start edit session.
   */
  public void createMember (String textDescription)
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException
  {
    com.ibm.as400.access.QSYSObjectPathName myObjectPath =
    new com.ibm.as400.access.QSYSObjectPathName(getLibraryName(), getObjectName(), "FILE");
    com.ibm.as400.access.SequentialFile myFile = new com.ibm.as400.access.SequentialFile(system, myObjectPath.getPath());
    myFile.setRecordFormat();
    myFile.addPhysicalFileMember(objectPath.getMemberName(),textDescription);
    // myAS400File.finalize() (to close any residual connection) ?
  }

  /** Check if the file has been opened */
  public boolean isOpen() {
    return as400File.isOpen();
  }

  /** Open the file on the host sytsem */
  public void open()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException
  {
    // Get record format if necessary
    getRecordFormat(); // This sets automatically if necessary.
    // Now we're definitely ready to open
    as400File.open(); // Opens read-write
  }

  /** Read in all records */
  public void readAll()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  java.beans.PropertyVetoException
  {
    // Set format, if necessary.
    getRecordFormat();

    // Read 'em.
    records = as400File.readAll();

    // We'll work on these records in a vector
    workingRecords = new java.util.Vector(java.util.Arrays.asList(records));
  }

  /** Write out all records.
   *  Caller should make sure file is open.
   */
  public void writeAll()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException
  {
    records = new com.ibm.as400.access.Record [0];
    records = (com.ibm.as400.access.Record []) workingRecords.toArray(records);
    as400File.write(records);
  }

  /** Clear the member and refill fresh will all records. */
  public void writeAllClearingMember ()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException,
  com.ibm.as400.access.ErrorCompletingRequestException,
  com.ibm.as400.access.ExtendedIllegalStateException,
  java.beans.PropertyVetoException
  {
    if (isOpen()) {
      as400File.close();
    }
    com.ibm.as400.access.CommandCall cc = new com.ibm.as400.access.CommandCall(system);
    cc.run("QSYS/CLRPFM FILE(" +  getLibraryName() + "/" + getObjectName() + ") MBR(" + getMemberName() + ")");
    // Show the messages (returned whether or not there was an error)
    com.ibm.as400.access.AS400Message[] messagelist = cc.getMessageList();
    for (int i=0; i < messagelist.length; i++)
    {
      // show each message
      System.out.println( messagelist[i].getText() );
    }
    as400File.open();
    writeAll();
  }

  /** Close the file. */
  public void close ()
  throws com.ibm.as400.access.AS400Exception,
  com.ibm.as400.access.AS400SecurityException,
  InterruptedException,
  java.io.IOException
  {
    if (isOpen()) {
      as400File.close();
    }
  }

  /********************************/
  /* Diagnostic and test routines */
  /********************************/
  /** Display record format of a SRCPFM. */
  public void displayRecordFormat() {
    // Get its record format.
    com.ibm.as400.access.RecordFormat rf = null;
    try {
      rf = getRecordFormat();
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (java.beans.PropertyVetoException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }

    System.out.println("Fields in record format for " + getIFSPath());
    System.out.println("Number of fields : " + Integer.toString(rf.getNumberOfFields()));
    String fieldNames [] = rf.getFieldNames();
    for (int i = 0; i < rf.getNumberOfFields(); i++) {
      com.ibm.as400.access.FieldDescription fd = rf.getFieldDescription(i);
      System.out.println(fieldNames[i]);
      System.out.println("   DDS Name: " + fd.getDDSName());
      System.out.println("   Data Type: " + fd.getDataType().getClass().getName());
      System.out.println("   Length: " + Integer.toString(fd.getLength()));
    }
  }

  /** Read in records and display */
  public static void testRead (String server, String lib, String file, String member) {

    // Create object representing extant file member.
    MEUFileMember fm = new MEUFileMember(new com.ibm.as400.access.AS400(server), lib, file, member);

    /* Print out the record format */
    fm.displayRecordFormat();

    // Read all in.
    try {
      fm.readAll();
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (java.beans.PropertyVetoException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }

    // Print out all records
    System.out.print(fm.recordsToString());

    // All done.
    try {
      fm.close();
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
    }
    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
    }
  }

  /** Read in records, modify, and write out */
  public static void testWrite (String server, String lib, String file, String member) {

    // Create object representing extant file member.
    MEUFileMember fm = new MEUFileMember(new com.ibm.as400.access.AS400(server), lib, file, member);

    /* Print out the record format */
    fm.displayRecordFormat();

    // Read all in.
    try {
      fm.readAll();
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (java.beans.PropertyVetoException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }

    // Print out all records
    System.out.print(fm.recordsToString());

    // All done.
    try {
      fm.close();
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
    }
    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
    }

    System.out.println("===");
    System.out.println("Now trying to write back all records due to one appended record.");
    System.out.println("===");

    try {
      com.ibm.as400.access.Record r = fm.getNewRecord
      (new java.math.BigDecimal("9999.00"),
      new java.math.BigDecimal("990609"),
      "0123456789012345678901234567890123456789012345678901234567890123456789");
      /* Testing if writing a shorter string leaves garbage in record. */
      fm.setSRCDTA(r, "/* This is the way we build our editor! Test record! */");
      fm.appendRecord(r);
      fm.writeAllClearingMember();
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
    }
    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
    }
    catch (java.beans.PropertyVetoException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
    catch (com.ibm.as400.access.ErrorCompletingRequestException e) {
      e.printStackTrace(System.err);
      System.exit(1);
    }
  }

  /** Diagnostic dump of all records to a string */
  public String recordsToString() {
    StringBuffer sb = new StringBuffer(records.length * (records[0].getRecordLength() + 1)); // We're going to append a linefeed
    for (int i = 0; i < records.length; i++) {
      sb.append(records[i].toString() + '\n');
    }
    return sb.toString();
  }

  /** Just a quick test of record access */
  public static void main (String argv[]) {

    // Make sure we have enuff args.
    if (argv.length < 4) {
      System.out.println ("Usage: MEUFileMember server lib file member");
      System.exit(0);
    }

    // Interpret our args.
    String server = argv[0];
    String lib = argv[1];
    String file = argv[2];
    String member = argv[3];

    String testType = "read";

    if (argv.length >= 5) {
      testType = argv[4].toLowerCase();
    }

    if (testType.equals("read")) {
      testRead(server, lib, file, member);
    }
    else if (testType.equals("write")) {
      testWrite(server, lib, file, member);
    }
    System.out.println(testType + " test completed");
    System.exit(0);
  }
}
