/*
 * MEUFrame.java
 * This is the top frame in which an edit session appears.
 * Created on July 29, 2000, 3:08 AM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

import java.io.*;

/** The top frame of the editor.
 * @author jax
 * @version 1.0
 */
public class MEUFrame extends javax.swing.JFrame {

  /** Top-level application controller */
  private MEU meu;

  /** Frame controller */
  private MEUController controller;

  /** Our icon image */
  private javax.swing.ImageIcon icon;

  /** Creates new form MEUFrame
   * and associates it with the top-level app controller
   */
  public MEUFrame(MEU m) {
    setMEU(m);
    controller = new MEUController(meu);
    initComponents ();
    setMnemonics ();
    orientFromProperties();
    pack ();
    initControl(); // This tells controller about its (now initialized) frame where it's safe to access components.
    initStatusBar(); // Because initControl must happen first for controller.getTextArea() to work.
    loadIcon("com/SoftWoehr/images/meu.jpg", "MEU Icon");
  }

  /** Creates new form MEUFrame,
   * associates it with the top-level app controller,
   * and attempts to load a file member.
   * If server is null, don't change server.
   * If ifsPath is null, don't load file member
   * @param m The MEU application controller instance.
   * @param serverName The name of the server to access for file members.
   * @param ifsPath The fully-qualified IFS pathname of the file member to edit.
   */
  public MEUFrame(MEU m, String serverName, String ifsPath) {
    this(m);
    if (null != serverName) {
      controller.setServer(serverName);
    }
    if (null != ifsPath) {
      if (controller.openEditSession(ifsPath)) {
        controller.setEditingEnabled(true);
      }
      else {
        MEUStockDialog.showErrorMessageDialog
        (this,
        "Couldn't open an editing session on " + ifsPath,
        "Problem loading file member");
      }
    }
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    jMenuBar1 = new javax.swing.JMenuBar ();
    fileMenu = new javax.swing.JMenu ();
    fileNewMenuItem = new javax.swing.JMenuItem ();
    fileOpenMenuItem = new javax.swing.JMenuItem ();
    jSeparator1 = new javax.swing.JSeparator ();
    fileSaveMenuItem = new javax.swing.JMenuItem ();
    fileSaveAsMenuItem = new javax.swing.JMenuItem ();
    fileSaveLocalMenuItem = new javax.swing.JMenuItem ();
    jSeparator2 = new javax.swing.JSeparator ();
    filePrintMenuItem = new javax.swing.JMenuItem ();
    jSeparator3 = new javax.swing.JSeparator ();
    fileCloseMenuItem = new javax.swing.JMenuItem ();
    fileExitMenuItem = new javax.swing.JMenuItem ();
    editMenu = new javax.swing.JMenu ();
    editCutMenuItem = new javax.swing.JMenuItem ();
    editCopyMenuItem = new javax.swing.JMenuItem ();
    editPasteMenuItem = new javax.swing.JMenuItem ();
    jSeparator4 = new javax.swing.JSeparator ();
    editInsertRowMenuItem = new javax.swing.JMenuItem ();
    editRemoveRowMenuItem = new javax.swing.JMenuItem ();
    jSeparator5 = new javax.swing.JSeparator ();
    editFindMenuItem = new javax.swing.JMenuItem ();
    editReplaceMenuItem = new javax.swing.JMenuItem ();
    systemMenu = new javax.swing.JMenu ();
    serverSystemMenuItem = new javax.swing.JMenuItem ();
    windowMenu = new javax.swing.JMenu ();
    newWindowMenuItem = new javax.swing.JMenuItem ();
    cascadeWindowMenuItem = new javax.swing.JMenuItem ();
    jSeparator6 = new javax.swing.JSeparator ();
    qcDemoWindowMenuItem = new javax.swing.JMenuItem ();
    settingsMenu = new javax.swing.JMenu ();
    saveSettingsMenuItem = new javax.swing.JMenuItem ();
    helpMenu = new javax.swing.JMenu ();
    helpHelpMenuItem = new javax.swing.JMenuItem ();
    jSeparator7 = new javax.swing.JSeparator ();
    aboutHelpMenuItem = new javax.swing.JMenuItem ();
    mEUHeaderPanel1 = new MEUHeaderPanel(controller);
    mEUTextArea1 = new MEUTextArea (controller);
    mEUStatusPanel1 = new MEUStatusPanel(controller);
    jMenuBar1.setToolTipText ("File operations");

      fileMenu.setToolTipText ("Open, create, save, print, close files");
      fileMenu.setText ("File");
  
        fileNewMenuItem.setToolTipText ("Create a new source file member");
        fileNewMenuItem.setText ("New");
        fileNewMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileNewMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileNewMenuItem);
        fileOpenMenuItem.setToolTipText ("Open an existing source file member");
        fileOpenMenuItem.setText ("Open");
        fileOpenMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileOpenMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileOpenMenuItem);
    
        fileMenu.add (jSeparator1);
        fileSaveMenuItem.setToolTipText ("Save the source file member");
        fileSaveMenuItem.setText ("Save");
        fileSaveMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileSaveMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileSaveMenuItem);
        fileSaveAsMenuItem.setToolTipText ("Save to a specific source file member");
        fileSaveAsMenuItem.setText ("Save As");
        fileSaveAsMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileSaveAsMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileSaveAsMenuItem);
        fileSaveLocalMenuItem.setToolTipText ("Save on the local file system a copy of the file currently being edited.");
        fileSaveLocalMenuItem.setText ("Save Locally");
        fileSaveLocalMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileSaveLocalMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileSaveLocalMenuItem);
    
        fileMenu.add (jSeparator2);
        filePrintMenuItem.setToolTipText ("Print the source file member");
        filePrintMenuItem.setText ("Print");
        filePrintMenuItem.setEnabled (false);
        filePrintMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            filePrintMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (filePrintMenuItem);
    
        fileMenu.add (jSeparator3);
        fileCloseMenuItem.setToolTipText ("Close the source file member");
        fileCloseMenuItem.setText ("Close");
        fileCloseMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileCloseMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileCloseMenuItem);
        fileExitMenuItem.setToolTipText ("Exit the editor and close the window");
        fileExitMenuItem.setText ("Exit");
        fileExitMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            fileExitMenuItemActionPerformed (evt);
          }
        }
        );
    
        fileMenu.add (fileExitMenuItem);
      jMenuBar1.add (fileMenu);
      editMenu.setToolTipText ("Change the text of the file being edited");
      editMenu.setText ("Edit");
  
        editCutMenuItem.setToolTipText ("Cut the selected text to the clipboard");
        editCutMenuItem.setText ("Cut");
        editCutMenuItem.setEnabled (false);
        editCutMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editCutMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editCutMenuItem);
        editCopyMenuItem.setToolTipText ("Copy selected text to clipboard");
        editCopyMenuItem.setText ("Copy");
        editCopyMenuItem.setEnabled (false);
        editCopyMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editCopyMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editCopyMenuItem);
        editPasteMenuItem.setToolTipText ("Paste text from the clipboard");
        editPasteMenuItem.setText ("Paste");
        editPasteMenuItem.setEnabled (false);
        editPasteMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editPasteMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editPasteMenuItem);
    
        editMenu.add (jSeparator4);
        editInsertRowMenuItem.setToolTipText ("Insert an empty row below the current row");
        editInsertRowMenuItem.setText ("Insert Row");
        editInsertRowMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editInsertRowMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editInsertRowMenuItem);
        editRemoveRowMenuItem.setToolTipText ("Remove the current row");
        editRemoveRowMenuItem.setText ("Remove Row");
        editRemoveRowMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editRemoveRowMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editRemoveRowMenuItem);
    
        editMenu.add (jSeparator5);
        editFindMenuItem.setToolTipText ("Find text");
        editFindMenuItem.setText ("Find");
        editFindMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editFindMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editFindMenuItem);
        editReplaceMenuItem.setToolTipText ("Replace text");
        editReplaceMenuItem.setText ("Replace");
        editReplaceMenuItem.setEnabled (false);
        editReplaceMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            editReplaceMenuItemActionPerformed (evt);
          }
        }
        );
    
        editMenu.add (editReplaceMenuItem);
      jMenuBar1.add (editMenu);
      systemMenu.setToolTipText ("Options relating to the server");
      systemMenu.setText ("System");
  
        serverSystemMenuItem.setToolTipText ("Set the AS/400 to which you wish this window to connect.");
        serverSystemMenuItem.setText ("Server");
        serverSystemMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            serverSystemMenuItemActionPerformed (evt);
          }
        }
        );
    
        systemMenu.add (serverSystemMenuItem);
      jMenuBar1.add (systemMenu);
      windowMenu.setToolTipText ("Manipulate multiple windows");
      windowMenu.setText ("Window");
  
        newWindowMenuItem.setToolTipText ("Open a new editing window");
        newWindowMenuItem.setText ("New Window");
        newWindowMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            newWindowMenuItemActionPerformed (evt);
          }
        }
        );
    
        windowMenu.add (newWindowMenuItem);
        cascadeWindowMenuItem.setToolTipText ("Cascade all windows");
        cascadeWindowMenuItem.setText ("Cascade");
        cascadeWindowMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            cascadeWindowMenuItemActionPerformed (evt);
          }
        }
        );
    
        windowMenu.add (cascadeWindowMenuItem);
    
        windowMenu.add (jSeparator6);
        qcDemoWindowMenuItem.setToolTipText ("Launch operations navigator");
        qcDemoWindowMenuItem.setText ("QCOpNav");
        qcDemoWindowMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            qcDemoWindowMenuItemActionPerformed (evt);
          }
        }
        );
    
        windowMenu.add (qcDemoWindowMenuItem);
      jMenuBar1.add (windowMenu);
      settingsMenu.setToolTipText ("View or change editor settings and defaults.");
      settingsMenu.setText ("Settings");
  
        saveSettingsMenuItem.setToolTipText ("Save your settings to the properties file they were loaded from.");
        saveSettingsMenuItem.setText ("Save");
        saveSettingsMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            saveSettingsMenuItemActionPerformed (evt);
          }
        }
        );
    
        settingsMenu.add (saveSettingsMenuItem);
      jMenuBar1.add (settingsMenu);
      helpMenu.setToolTipText ("Help and About");
      helpMenu.setText ("Help");
  
        helpHelpMenuItem.setToolTipText ("Currently, the only help is the flyover text.");
        helpHelpMenuItem.setText ("Help");
        helpHelpMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            helpHelpMenuItemActionPerformed (evt);
          }
        }
        );
    
        helpMenu.add (helpHelpMenuItem);
    
        helpMenu.add (jSeparator7);
        aboutHelpMenuItem.setToolTipText ("About SoftWoehr MEU and QCOpNav");
        aboutHelpMenuItem.setText ("About");
        aboutHelpMenuItem.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            aboutHelpMenuItemActionPerformed (evt);
          }
        }
        );
    
        helpMenu.add (aboutHelpMenuItem);
      jMenuBar1.add (helpMenu);
    setTitle ("SoftWoehr MEU 1.0");
    setResizable (false);
    setDefaultCloseOperation (javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
    addWindowListener (new java.awt.event.WindowAdapter () {
      public void windowClosing (java.awt.event.WindowEvent evt) {
        exitForm (evt);
      }
    }
    );

    mEUHeaderPanel1.setLayout (new java.awt.GridLayout (2, 3));


    getContentPane ().add (mEUHeaderPanel1, java.awt.BorderLayout.NORTH);

    mEUTextArea1.setToolTipText ("Edit AS/400 source physical file members in this window");
    mEUTextArea1.setCaretColor (mEUTextArea1.getCaretColorFromProperty());
    mEUTextArea1.setForeground (mEUTextArea1.getForegroundColorFromProperty());
    mEUTextArea1.setBackground (mEUTextArea1.getBackgroundColorFromProperty ());


    getContentPane ().add (mEUTextArea1, java.awt.BorderLayout.CENTER);

    mEUStatusPanel1.setLayout (new java.awt.GridLayout (1, 3));


    getContentPane ().add (mEUStatusPanel1, java.awt.BorderLayout.SOUTH);

    setJMenuBar (jMenuBar1);

  }//GEN-END:initComponents

  private void aboutHelpMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutHelpMenuItemActionPerformed
    // Add your handling code here:
    MEUStockDialog.showAboutSoftWoehrDialog(icon);
  }//GEN-LAST:event_aboutHelpMenuItemActionPerformed

  private void helpHelpMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpHelpMenuItemActionPerformed
    // Add your handling code here:
    MEUStockDialog.showHelpDialog(icon);
  }//GEN-LAST:event_helpHelpMenuItemActionPerformed

  private void qcDemoWindowMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_qcDemoWindowMenuItemActionPerformed
    // Add your handling code here:
    meu.eventQCDemo(evt);
  }//GEN-LAST:event_qcDemoWindowMenuItemActionPerformed

  private void editRemoveRowMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editRemoveRowMenuItemActionPerformed
    // Add your handling code here:
    controller.menuRemoveRow(evt);
  }//GEN-LAST:event_editRemoveRowMenuItemActionPerformed

  private void editInsertRowMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editInsertRowMenuItemActionPerformed
    // Add your handling code here:
    controller.menuInsertRow(evt);
  }//GEN-LAST:event_editInsertRowMenuItemActionPerformed

  private void fileSaveLocalMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileSaveLocalMenuItemActionPerformed
    // Add your handling code here:
    controller.menuSaveLocal(evt);
  }//GEN-LAST:event_fileSaveLocalMenuItemActionPerformed

  private void serverSystemMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_serverSystemMenuItemActionPerformed
    // Add your handling code here:
    controller.menuSetServer(evt);
  }//GEN-LAST:event_serverSystemMenuItemActionPerformed

  private void saveSettingsMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveSettingsMenuItemActionPerformed
    // Add your handling code here:
    meu.storeProperties(null);
  }//GEN-LAST:event_saveSettingsMenuItemActionPerformed

  private void cascadeWindowMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cascadeWindowMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    meu.cascade();
  }//GEN-LAST:event_cascadeWindowMenuItemActionPerformed

  private void newWindowMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newWindowMenuItemActionPerformed
    // Refer request for new frame to the top-level application controller.
    // Open just southeast of parent.
    // /* Debug */ System.out.println(evt.paramString());

    // Defaults in case property throws
    int xInc = 20;
    int yInc = 20;

    // Where is parent?
    java.awt.Point p = getLocationOnScreen();

    // Get properties
    try {
      xInc = meu.getIntProperty("com.SoftWoehr.JTOpenContrib.MEU.window.cascade.xincrement");
      yInc = meu.getIntProperty("com.SoftWoehr.JTOpenContrib.MEU.window.cascade.yincrement");

    }
    catch (MEUProperties.NoSuchPropertyException e) {
      e.printStackTrace(System.err);
    }
    catch (MEUProperties.InvalidIntPropertyException e) {
      e.printStackTrace(System.err);
    }

    // Choose position for new frame and open.
    p.translate(xInc, yInc);
    meu.newFrame(p);
  }//GEN-LAST:event_newWindowMenuItemActionPerformed

  private void editReplaceMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editReplaceMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
  }//GEN-LAST:event_editReplaceMenuItemActionPerformed

  private void editFindMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editFindMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    controller.menuSearch(evt);
  }//GEN-LAST:event_editFindMenuItemActionPerformed

  private void editPasteMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editPasteMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
  }//GEN-LAST:event_editPasteMenuItemActionPerformed

  private void editCutMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editCutMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
  }//GEN-LAST:event_editCutMenuItemActionPerformed

  private void editCopyMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editCopyMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
  }//GEN-LAST:event_editCopyMenuItemActionPerformed

  private void fileExitMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileExitMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    meu.requestShutdown(this);
  }//GEN-LAST:event_fileExitMenuItemActionPerformed

  private void fileCloseMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileCloseMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    iWantToClose();
  }//GEN-LAST:event_fileCloseMenuItemActionPerformed

  private void filePrintMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_filePrintMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
  }//GEN-LAST:event_filePrintMenuItemActionPerformed

  private void fileSaveAsMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileSaveAsMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    controller.menuSaveAs(evt);
  }//GEN-LAST:event_fileSaveAsMenuItemActionPerformed

  private void fileSaveMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileSaveMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    controller.menuSave(evt);
  }//GEN-LAST:event_fileSaveMenuItemActionPerformed

  private void fileOpenMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileOpenMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    controller.menuOpenFileMember(evt);
  }//GEN-LAST:event_fileOpenMenuItemActionPerformed

  private void fileNewMenuItemActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileNewMenuItemActionPerformed
    // Add your handling code here:
    // /* Debug */ System.out.println(evt.paramString());
    controller.menuNewFileMember(evt);
  }//GEN-LAST:event_fileNewMenuItemActionPerformed

  /** Close the frame by checking with the associated MEUController frame
   * controller. The MEUController checks with the top-level controller MEU
   * which decides if app exits
   */
  private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
    // /* Debug */ System.out.println(evt.paramString());
    iWantToClose();
  }//GEN-LAST:event_exitForm

  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JMenuBar jMenuBar1;
  private javax.swing.JMenu fileMenu;
  private javax.swing.JMenuItem fileNewMenuItem;
  private javax.swing.JMenuItem fileOpenMenuItem;
  private javax.swing.JSeparator jSeparator1;
  private javax.swing.JMenuItem fileSaveMenuItem;
  private javax.swing.JMenuItem fileSaveAsMenuItem;
  private javax.swing.JMenuItem fileSaveLocalMenuItem;
  private javax.swing.JSeparator jSeparator2;
  private javax.swing.JMenuItem filePrintMenuItem;
  private javax.swing.JSeparator jSeparator3;
  private javax.swing.JMenuItem fileCloseMenuItem;
  private javax.swing.JMenuItem fileExitMenuItem;
  private javax.swing.JMenu editMenu;
  private javax.swing.JMenuItem editCutMenuItem;
  private javax.swing.JMenuItem editCopyMenuItem;
  private javax.swing.JMenuItem editPasteMenuItem;
  private javax.swing.JSeparator jSeparator4;
  private javax.swing.JMenuItem editInsertRowMenuItem;
  private javax.swing.JMenuItem editRemoveRowMenuItem;
  private javax.swing.JSeparator jSeparator5;
  private javax.swing.JMenuItem editFindMenuItem;
  private javax.swing.JMenuItem editReplaceMenuItem;
  private javax.swing.JMenu systemMenu;
  private javax.swing.JMenuItem serverSystemMenuItem;
  private javax.swing.JMenu windowMenu;
  private javax.swing.JMenuItem newWindowMenuItem;
  private javax.swing.JMenuItem cascadeWindowMenuItem;
  private javax.swing.JSeparator jSeparator6;
  private javax.swing.JMenuItem qcDemoWindowMenuItem;
  private javax.swing.JMenu settingsMenu;
  private javax.swing.JMenuItem saveSettingsMenuItem;
  private javax.swing.JMenu helpMenu;
  private javax.swing.JMenuItem helpHelpMenuItem;
  private javax.swing.JSeparator jSeparator7;
  private javax.swing.JMenuItem aboutHelpMenuItem;
  private com.SoftWoehr.JTOpenContrib.MEU.MEUHeaderPanel mEUHeaderPanel1;
  private com.SoftWoehr.JTOpenContrib.MEU.MEUTextArea mEUTextArea1;
  private com.SoftWoehr.JTOpenContrib.MEU.MEUStatusPanel mEUStatusPanel1;
  // End of variables declaration//GEN-END:variables

  private void setMnemonics () {
    // Set file menu mnemonics
    fileMenu.setMnemonic(java.awt.event.KeyEvent.VK_F);
    fileNewMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_N);
    fileOpenMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_O);
    fileSaveMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_S);
    fileSaveAsMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_A);
    fileSaveLocalMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_C);
    filePrintMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_P);
    fileCloseMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_L);
    fileExitMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_X);

    // Set edit menu mnemonics
    editMenu.setMnemonic(java.awt.event.KeyEvent.VK_E);
    editCutMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_T);
    editCopyMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_C);
    editPasteMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_P);
    editInsertRowMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_N);
    editRemoveRowMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_O);
    editFindMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_I);
    editReplaceMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_R);

    // Set data menu mnemonics
    systemMenu.setMnemonic(java.awt.event.KeyEvent.VK_Y);
    serverSystemMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_R);

    // Set window menu mnemonics
    windowMenu.setMnemonic(java.awt.event.KeyEvent.VK_W);
    newWindowMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_N);
    cascadeWindowMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_C);
    qcDemoWindowMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_Q);

    // Set Settings menu mnemonics
    settingsMenu.setMnemonic(java.awt.event.KeyEvent.VK_G);
    saveSettingsMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_S);

    helpMenu.setMnemonic(java.awt.event.KeyEvent.VK_H);
    helpHelpMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_L);
    aboutHelpMenuItem.setMnemonic(java.awt.event.KeyEvent.VK_A);
  }

  /** Set up the frame controller */
  private void initControl () {
    controller.setFrame(this);
    controller.setDefaultServer(); // This needs the frame so happens here.
    // mEUHeaderPanel1.setController (controller); // Now set in ctor
    // mEUTextArea1.setController (controller); // Now set in ctor
  }

  /** Set the top-level application controller */
  private void setMEU (MEU m) {
    meu = m;
  }

  /** Get the top-level application controller */
  public MEU getMEU () {
    return meu;
  }

  /** Get the current font size in the text component */
  public int queryFontSize() {
    return mEUTextArea1.getFont().getSize();
  }

  /** Change the font and resize */
  public void resetFont (java.awt.Font f) {
    mEUTextArea1.resetFont(f);
    mEUHeaderPanel1.setRuleFont(f);
    pack();
    // repaint();
  }

  /** Change the number of columns and rows of editable text */
  public void resetSize (int columns, int rows) {
    mEUTextArea1.resetSize(columns, rows);
    mEUHeaderPanel1.resetColumns(columns);
    pack();
  }

  /** Resize self from the properties; called in ctor */
  private void orientFromProperties() {
    int fontsize = 16;
    int columns  = 95;
    int rows     = 25;
    try {
      fontsize = meu.getIntProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.fontsize");
    }
    catch (MEUProperties.NoSuchPropertyException e) {
      MEUStockDialog.showExceptionMessageDialog(null, "Property doesn't exist", e);
    }
    catch (MEUProperties.InvalidIntPropertyException e) {
      MEUStockDialog.showExceptionMessageDialog(null, "Invalid integer property in fontsize.", e);
    }
    if (fontsize > 1 & fontsize < 73) {
      resetFont(new java.awt.Font("Monospaced", java.awt.Font.PLAIN, fontsize));
    }
    else {
      MEUStockDialog.showPropertyErrorMessageDialog
      (null,
      "Font size must be between 2 and 72.",
      "com.SoftWoehr.JTOpenContrib.MEU.window.edittext.fontsize",
      Integer.toString(fontsize));
    }
    try {
      columns = meu.getIntProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.columns");
      rows    = meu.getIntProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.rows");
    }
    catch (MEUProperties.NoSuchPropertyException e) {
      MEUStockDialog.showExceptionMessageDialog(null, "Invalid integer property edittext columns or rows", e);
    }
    catch (MEUProperties.InvalidIntPropertyException e) {
      MEUStockDialog.showExceptionMessageDialog(null, "Invalid integer property in edittext columns or rows", e);
    }
    if (columns > 25 & columns < 128)  // Minimum 6 + 1 + 12 + 1 + 6
    {
      if (rows > 6 & rows < 81) {
        resetSize(columns, rows);
      }
      else {
        MEUStockDialog.showPropertyErrorMessageDialog
        (null,
        "Rows must be between 7 and 80 inclusive",
        "com.SoftWoehr.JTOpenContrib.MEU.window.edittext.rows",
        Integer.toString(rows));
      }
    }
    else {
      MEUStockDialog.showPropertyErrorMessageDialog
      (null,
      "Columns must be between 26 and 127 inclusive",
      "com.SoftWoehr.JTOpenContrib.MEU.window.edittext.columns",
      Integer.toString(columns));
    }
  }

  /*************************************/
  /* Support for actions by controller */
  /*************************************/

  /** Get the text area which is the editor region */
  public MEUTextArea getTextArea() {
    return mEUTextArea1;
  }

  /** Get the text area which is the editor region */
  public MEUStatusPanel getStatusBar() {
    return mEUStatusPanel1;
  }

  public void initStatusBar() {
    mEUStatusPanel1.statusAll();
  }

  /** Show text */
  // public void display(String text) {
  // mEUTextArea1.setText(text);
  // }

  //  /* Change editability */
  //  public void setEditingEnabled (boolean enabled) {
  //    mEUTextArea1.setEnabled(enabled);
  //  }

  /** Hop focus between command line and text area */
  public void toggleFocus () {
    if (mEUTextArea1.hasFocus()) {
      mEUHeaderPanel1.commandLineFocus();
    }
    else { // Don't throw focus if disabled
      if (mEUTextArea1.isEnabled()) {
        mEUTextArea1.requestFocus();
      }
    }
  }

  /************/
  /* The Icon */
  /************/

  private boolean loadIcon (String iconPath, String description) {
    boolean result = false;
    java.net.URL iconURL = ClassLoader.getSystemResource(iconPath);
    if (iconURL != null) {
      icon = new javax.swing.ImageIcon(iconURL, description);
      if (null != icon)
      {
        setIconImage(icon.getImage());
        result = true;
      }
    }
    return result;
  }

  /************/
  /* Shutdown */
  /************/

  /** Try to close */
  public void iWantToClose() {
    if (controller.closeRequested(this)) {
      cleanup();
      dispose();
    }
  }

  /** Forward shutdown request to MEUontroller, the frame controller.
   * This is called by MEU, the application controller, when the
   * user pulls the "Exit" menu option, so that MEU can decide
   * if every frame agrees to exit.
   */
  public boolean requestShutdown(MEUFrame f) {
    // Debug
    // System.out.println("Shutdown requested frame " + this.toString());

    return controller.requestShutdown(f);
  }

  /** Clean up connection stuff -- forward to MEUController. */
  public void cleanup() {
    /* Anything to do here for the window stuff itself?
     * The MEU application controller will dispose().
     */
    // Debug
    //  System.out.println("Cleanup requested MEUFrame " + this.toString());

    // Forward
    controller.cleanup();

  }
}
