/*
 * MEUHeaderPanel.java
 * This is the panel holding the widgets at the top of an MEU window.
 * Created on July 29, 2000, 2:26 AM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

/** The panel which contains the command line and the format rule.
 * @author jax
 * @version 1.0
 */
public class MEUHeaderPanel extends javax.swing.JPanel {

  private MEUController controller;

  /** Creates new form MEUHeaderPanel */
  public MEUHeaderPanel() {
    initComponents ();
  }

  /** Creates new form MEUHeaderPanel and assigns controller. */
  public MEUHeaderPanel(MEUController c) {
    setController(c);
    initComponents();
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    mEUCommandLine1 = new MEUCommandLine(controller);
    formatLabel = new javax.swing.JLabel ();
    setLayout (new java.awt.GridLayout (2, 1));



    add (mEUCommandLine1);

    formatLabel.setText (" FMT **  ...+... 1 ...+... 2 ...+... 3 ...+... 4 ...+... 5 ...+... 6 ...+... 7 ...+... 8 ");
    formatLabel.setToolTipText ("Format rule");
    formatLabel.setHorizontalAlignment (javax.swing.SwingConstants.LEFT);
    formatLabel.setFont (new java.awt.Font ("Monospaced", 0, 16));


    add (formatLabel);

  }//GEN-END:initComponents


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private com.SoftWoehr.JTOpenContrib.MEU.MEUCommandLine mEUCommandLine1;
  private javax.swing.JLabel formatLabel;
  // End of variables declaration//GEN-END:variables

  /** Set controller (at ctor time) */
  protected void setController (MEUController c) {
    controller = c;
    // mEUCommandLine1.setController(c); // Not needed, done in command line ctor
  }

  /** Set the font in which the format rule displays */
  public void setRuleFont (java.awt.Font f) {
    formatLabel.setFont(f);
  }

  /** Set the text which the format rule displays */
  public void setRuleText (String text) {
    formatLabel.setText (text);
  }

  private String genFormatRule (int columns) {
    String formatHeading = " FMT ** ";
    StringBuffer sb = new StringBuffer(formatHeading);
    for(int i = 0; i + formatHeading.length() < columns - " 000000".length(); i++) {

      /* Count off the length which sits over the editable text,
       *  appending appropriate format rule chars.
       */
      switch (i%10) {
        case 9 :
        sb.append(Integer.toString(((i+1) / 10) % 10));
        break;

        case 1:
        case 2:
        case 3:
        case 5:
        case 6:
        case 7:
        sb.append('.');
        break;

        case 4:
        sb.append('+');
        break;

        default:
        sb.append(' ');
      }
    }
    return sb.toString();
  }

  /** Redraw the rule text to match number of columns */
  public void resetColumns (int columns) {
    // Redraw the rule
    formatLabel.setText(genFormatRule(columns));
  }

  /** Tell the command line to accept focus */
  public void commandLineFocus() {
    mEUCommandLine1.commandLineFocus();
  }
}
