/*
 * MEUProperties.java
 * Represents the properties loaded and saved by MEU.
 * Created on August 2, 2000, 11:48 AM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

/** A Properties entity for MEU
 * @author jax
 * @version 1.0
 */
public class MEUProperties extends java.util.Properties {

  private static java.util.Properties defaultProperties;

  private static final String PROPERTIES_HEADER = "SoftWoehr MEU " + MEU.MEU_VERSION + " Properties";

  /** To respond to requests */
  public class NoSuchPropertyException extends java.lang.Exception {
    NoSuchPropertyException(String property) {
      super("Unknown property: " + property);
    }
  }

  /** To respond to requests */
  public class InvalidIntPropertyException extends java.lang.Exception {
    InvalidIntPropertyException(String property) {
      super("Invalid integer property: " + property);
    }
  }

  /** Create the defaults */
  static {
    defaultProperties = new java.util.Properties ();

    /* Set all the defaults */
    // defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.system.server.user", null);
    // defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.system.server.servername", null);
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.system.server.ifsdirectory", "/");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.cascade.xincrement" , "20");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.cascade.yincrement" , "20");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.color.foreground"   , "green");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.color.background"   , "black");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.color.cursor"       , "green");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.columns"   , "95");

    // Columns is (SRCSEQ + '.' + ' ' + SRCDTA + ' ' + SRCDAT)
    //                  6 +   1 +   1 +     80 +   1 +      6 == 95
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.columns"   , "95");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.fontsize"  , "12");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.rows"      , "25");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.numrowsdisplayed", "25");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.edittext.toprowdisplayed" , "-1");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.open.xlocation"     , "20");
    defaultProperties.setProperty("com.SoftWoehr.JTOpenContrib.MEU.window.open.ylocation"     , "20");
  }

  /** Creates new MEUProps */
  public MEUProperties() {
    super(defaultProperties);
  }

  /** Load properties at creation
   * @param filename Properties file on local file system.
   */
  public MEUProperties(String filename) {
    this();
    try {
      loadFromFile(filename);
    }

    catch (java.io.FileNotFoundException e) {
      e.printStackTrace(System.err);
    }

    catch (java.io.IOException e) {
      e.printStackTrace(System.err);
    }
  }

  /** Reload saved properties from a local file
   * @param filename Local file to fetch property strings from
   * @throws FileNotFoundException Couldn't find that path.
   * @throws IOException Couldn't read for some reason.
   */
  public void loadFromFile (String filename)
  throws java.io.FileNotFoundException, java.io.IOException
  {
    java.io.FileInputStream fi = new java.io.FileInputStream(filename);
    load(fi);
    fi.close();
  }

  /** Save the properties
   * @param filename File on local file system to store props. to.
   * @throws FileNotFoundException Couldn't find that path.
   * @throws IOException Couldn't write for some reason.
   */
  public void storeToFile (String filename)
  throws java.io.FileNotFoundException, java.io.IOException
  {
    java.io.FileOutputStream fo = new java.io.FileOutputStream(filename);
    store(fo, PROPERTIES_HEADER);
  }

  /** Get a specific int property.
   * @param property Name of property.
   * @throws NoSuchPropertyException No such.
   * @throws InvalidIntPropertyException Wasn't a base10 int.
   * @return Int value base 10 of property value
   */
  public int getIntProperty (String property)
  throws NoSuchPropertyException, InvalidIntPropertyException
  {
    int value = 0;
    String prop = getProperty(property);
    if (null == prop)
    {
      throw new NoSuchPropertyException(property);
    }
    try {
      value = Integer.parseInt(prop);
    }
    catch (NumberFormatException e) {
      throw new InvalidIntPropertyException(property);
    }
    return value;
  }
}
