/*
 * MEUStatusPanel.java
 * Holds various status fields and widgets.
 * Created on August 12, 2000, 5:15 PM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

/** Status panel to display info to MEU user.
 * @author jax
 * @version 1.0
 */
public class MEUStatusPanel extends javax.swing.JPanel {

  private MEUController controller;

  /** Compose status */
  private StringBuffer changedIndicator;
  private StringBuffer pendingIndicator;

  /** Creates new form MEUStatusPanel */
  /** Not used except by NetBeans design tool */
  public MEUStatusPanel() {
    initComponents ();
  }

  /** Creates new form MEUStatusPanel
   * @param c The MEUController for the StatusPanel.
   */
  public MEUStatusPanel(MEUController c) {
    setController(c);
    initComponents ();
    initChangedIndicator();
    initPendingIndicator();
    initMessage();
  }

  /** Set the controller */
  private void setController(MEUController c) {
    controller = c;
  }

  /** Set up the buffer for status changedIndicators */
  private void initChangedIndicator() {
    changedIndicator = new StringBuffer("UNCHANGED");
  }

  /** Set pending command indicator */
  private void initPendingIndicator() {
    pendingIndicator = new StringBuffer("  ");
  }

  /** Set up message to blank */
  private void initMessage() {
    statusMessage(" ");
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    changedIndicatorLabel = new javax.swing.JLabel ();
    pendingLabel = new javax.swing.JLabel ();
    messageTextField = new javax.swing.JTextField ();
    insertButton = new javax.swing.JButton ();
    setLayout (new java.awt.GridLayout (1, 1));
    setBorder (new javax.swing.border.TitledBorder("Status"));

    changedIndicatorLabel.setBorder (new javax.swing.border.TitledBorder("Changed"));
    changedIndicatorLabel.setText ("      ");
    changedIndicatorLabel.setToolTipText ("Shows status of edit session");
    changedIndicatorLabel.setFont (new java.awt.Font ("Monospaced", 1, 12));
    changedIndicatorLabel.setMaximumSize (new java.awt.Dimension(32767, 32767));


    add (changedIndicatorLabel);

    pendingLabel.setBorder (new javax.swing.border.TitledBorder("Pending"));
    pendingLabel.setToolTipText ("Indicates line commands pending");
    pendingLabel.setFont (new java.awt.Font ("Monospaced", 1, 12));


    add (pendingLabel);

    messageTextField.setBorder (new javax.swing.border.TitledBorder("Messages"));
    messageTextField.setEditable (false);
    messageTextField.setFont (new java.awt.Font ("Monospaced", 0, 12));


    add (messageTextField);

    insertButton.setToolTipText ("Click to toggle insert mode / overwrite mode.");
    insertButton.setHorizontalTextPosition (javax.swing.SwingConstants.CENTER);
    insertButton.setText ("OVR");
    insertButton.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        insertButtonActionPerformed (evt);
      }
    }
    );


    add (insertButton);

  }//GEN-END:initComponents

  private void insertButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_insertButtonActionPerformed
    // Add your handling code here:
    controller.actionToggleInserting(evt);
    controller.textAreaFocus(); // because button grabs focus
  }//GEN-LAST:event_insertButtonActionPerformed

  /** Allow controller to flag button text */
  public void statusInsertButton() {
    statusInsertButton(controller.getInserting());
  }

  /** Flag button text
   * @param status true - insert
   * false - overwrite
   */
  public void statusInsertButton(boolean status) {
    if(status) {
      insertButton.setText("INS");
    }
    else {
      insertButton.setText("OVR");
    }
  }

  /** Make up the changed indicator text */
  protected void composeChangedIndicator () {
    if (controller.isChanged()) {
      changedIndicator.replace(0, changedIndicator.length(), "  CHANGED");
    }
    else {
      changedIndicator.replace(0, changedIndicator.length(), "UNCHANGED");
    }
  }

  /** Update the status indicator */
  public void statusChangedIndicator () {
    composeChangedIndicator();
    changedIndicatorLabel.setText(changedIndicator.toString());
  }

  /** Set a status message
   * @param message The message
   */
  public void statusMessage(String message)  {
    if (null == message) {
      messageTextField.setText(" ");
    }
    else {
      messageTextField.setText (message);
      // messageTextField.setSize
      // (java.lang.Math.min(80, message.length()) * messageTextField.getColumnWidth(),
      // messageTextField.getRowHeight()
      // );
    }
  }

  /** Update all status */
  public void statusAll() {
    statusAll(" ");
  }

  /** Update all status
   * @param message The message or null
   */
  public void statusAll(String message) {
    statusInsertButton();
    statusChangedIndicator ();
    statusMessage(message);
  }


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JLabel changedIndicatorLabel;
  private javax.swing.JLabel pendingLabel;
  private javax.swing.JTextField messageTextField;
  private javax.swing.JButton insertButton;
  // End of variables declaration//GEN-END:variables

}
