/*
 * MEUStockDialog.java
 * Various dialogs we use.
 * Created on August 2, 2000, 5:50 PM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU;

/** MEU stock dialogs, all static methods
 * @author jax
 * @version 1.0
 */
public class MEUStockDialog extends Object {

  /** Creates new MEUStockDialog */
  protected MEUStockDialog() {
  }

  /** Inform the user in a dialog
   * @param frame Parent frame
   * @param message Text message
   * @param title Text title
   */
  public static void showInfoMessageDialog (javax.swing.JFrame frame, String message, String title) {
    javax.swing.JOptionPane.showMessageDialog
    (frame, message, title , javax.swing.JOptionPane.INFORMATION_MESSAGE);
  }

  /** Show some error in a dialog
   * @param frame Parent frame
   * @param message Text message
   * @param title Text title
   */
  public static void showErrorMessageDialog(javax.swing.JFrame frame, String message, String title) {
    javax.swing.JOptionPane.showMessageDialog
    (frame,
    message,
    title,
    javax.swing.JOptionPane.ERROR_MESSAGE);
  }

  /** Show warning in a dialog
   * @param frame Parent frame
   * @param message The warning message
   * @param title The title
   */
  public static void showWarningMessageDialog(javax.swing.JFrame frame, String message, String title) {
    javax.swing.JOptionPane.showMessageDialog
    (frame,
    message,
    title,
    javax.swing.JOptionPane.WARNING_MESSAGE);
  }

  /** Inform user of problem executing a command
   * @param frame Parent frame
   * @param message Text message
   * @param command Text command as issued by user
   */
  public static void showCommandErrorDialog (javax.swing.JFrame frame, String message, String command) {
    showErrorMessageDialog(frame, message, "Problem executing: " + command);
  }

  /** Inform the user in response to a command
   * @param frame Parent frame
   * @param message Text message
   * @param command Text command as issued by user
   */
  public static void showCommandInfoMessageDialog (javax.swing.JFrame frame, String message, String command) {
    javax.swing.JOptionPane.showMessageDialog
    (frame, message, "Command: " + command, javax.swing.JOptionPane.INFORMATION_MESSAGE);
  }

  /** Inform the user of a problem
   * @param frame Parent frame
   * @param message Text message
   * @param exception Exception referenced by this dialog

   */
  public static void showExceptionMessageDialog (javax.swing.JFrame frame, String message, Exception exception) {
    javax.swing.JOptionPane.showMessageDialog
    (frame, message, "Exception: " + exception.toString(), javax.swing.JOptionPane.ERROR_MESSAGE);
  }

  /** Inform the user of a problem
   * @param frame Parent frame
   * @param message Text message
   * @param property Property name
   * @param value The value for the property.
   */
  public static void showPropertyErrorMessageDialog (javax.swing.JFrame frame, String message, String property, String value) {
    javax.swing.JOptionPane.showMessageDialog
    (frame, message, "Property error: " + property + "=" + value, javax.swing.JOptionPane.ERROR_MESSAGE);
  }

  /** Offer a file open dialog.
   * @param sys AS/400 system to use
   * @param ifsdirectory Directory to search
   * @param frame Parent frame
   * @return Filepath or null
   */
  public static String showOpenFileMemberDialog(com.ibm.as400.access.AS400 sys, String ifsdirectory, javax.swing.JFrame frame) {

    String result = null;
    com.ibm.as400.vaccess.IFSFileDialog fd = null;

    if (null != sys) {
      fd = new com.ibm.as400.vaccess.IFSFileDialog(frame, "File Member Open on " + sys.getSystemName(), sys);
      com.ibm.as400.vaccess.FileFilter[] filterList = {
        new com.ibm.as400.vaccess.FileFilter("All files (*.*)", "*.*"),
        new com.ibm.as400.vaccess.FileFilter("Members (*.MBR)", "*.MBR")
      };

      fd.setFileFilter(filterList, 1);
      fd.setDirectory(ifsdirectory);

      if (fd.showDialog() == com.ibm.as400.vaccess.IFSFileDialog.OK)
      {
        result = fd.getAbsolutePath();    // get fully qualified file
      }
    }
    fd.dispose();
    return result;
  }

  /** Offer a file save as dialog.
   * @param sys AS/400 system to use
   * @param ifsdirectory Directory to search
   * @param frame Parent frame
   * @return Filepath or null.
   */
  public static String showSaveAsFileMemberDialog(com.ibm.as400.access.AS400 sys, String ifsdirectory, javax.swing.JFrame frame) {

    String result = null;
    com.ibm.as400.vaccess.IFSFileDialog fd = null;

    if (null != sys) {
      fd = new com.ibm.as400.vaccess.IFSFileDialog(frame, "File Member Save As on " + sys.getSystemName(), sys);
      com.ibm.as400.vaccess.FileFilter[] filterList = {
        new com.ibm.as400.vaccess.FileFilter("All files (*.*)", "*.*"),
        new com.ibm.as400.vaccess.FileFilter("Members (*.MBR)", "*.MBR")
      };

      fd.setFileFilter(filterList, 1);
      fd.setDirectory(ifsdirectory);

      if (fd.showDialog() == com.ibm.as400.vaccess.IFSFileDialog.OK)
      {
        result = fd.getAbsolutePath();    // get fully qualified file
      }
    }
    fd.dispose();
    return result;
  }

  /** Offer a file save dialog for local disk.
   * @param localDirectory Dir to search
   * @param frame Parent frame
   * @return Local file path or null.
   */
  public static java.io.File showSaveLocalFileMemberDialog(String localDirectory, javax.swing.JFrame frame) {
    java.io.File result = null;

    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser(new java.io.File (localDirectory));

    chooser.setFileFilter(new javax.swing.filechooser.FileFilter (){
      /**
       * Return true if this file should be shown in the directory pane,
       * false if it shouldn't.
       *
       * Files that begin with "." are ignored.
       */
      public boolean accept(java.io.File f) {
        boolean rc = false;
        if(f != null) {
          if(f.isDirectory()) {
            rc = true;
          }
          String extension = getExtension(f);
          if(extension != null) {
            if (extension.equals("txt")) {
              rc =  true;
            }
          }
        }
        return rc;
      }

      /**
       * Return the extension portion of the file's name .
       *       */
      public String getExtension(java.io.File f) {
        if(f != null) {
          String filename = f.getName();
          int i = filename.lastIndexOf('.');
          if(i>0 && i<filename.length()-1) {
            return filename.substring(i+1).toLowerCase();
          };
        }
        return null;
      }

      public String getDescription () {
        return "Text files (*.txt)";
      }
    });

    int userResponse = chooser.showSaveDialog(frame);

    if(userResponse == javax.swing.JFileChooser.APPROVE_OPTION) {
      result = chooser.getSelectedFile();
    }
    else {
      showInfoMessageDialog (frame, "Operation cancelled.", "File Was Not Saved");
    }
    return result;
  }

  /** Ask if user wants to overwrite existing file.
   * @param sys System to use
   * @param ifspath Pathname
   * @param frame Parent frame
   * @return javax.swing.JOptionPane mask representing button press
   */
  public static int showOverwriteFileMemberDialog (com.ibm.as400.access.AS400 sys, String ifspath, javax.swing.JFrame frame) {
    int result = javax.swing.JOptionPane.CANCEL_OPTION;

    if (null != sys) {
      result =
      javax.swing.JOptionPane.showConfirmDialog
      (frame,
      "File member " + sys.getSystemName() + ":" + ifspath + " exists, overwrite?",
      "Save As - Overwrite File Member?",
      javax.swing.JOptionPane.YES_NO_CANCEL_OPTION,
      javax.swing.JOptionPane.QUESTION_MESSAGE
      );
    }
    return result;
  }

  /** Ask if user wants to create new file member.
   * @param sys System to use
   * @param ifspath Full pathname
   * @param frame Parent frame
   * @return javax.swing.JOptionPane mask representing button press
   */
  public static int showCreateFileMemberDialog (com.ibm.as400.access.AS400 sys, String ifspath, javax.swing.JFrame frame) {
    int result = javax.swing.JOptionPane.CANCEL_OPTION;

    if (null != sys) {
      result =
      javax.swing.JOptionPane.showConfirmDialog
      (frame,
      "File member " + sys.getSystemName() + ":" + ifspath + " does not exist, create?",
      "Save As - Create File?",
      javax.swing.JOptionPane.YES_NO_CANCEL_OPTION,
      javax.swing.JOptionPane.QUESTION_MESSAGE
      );
    }
    return result;
  }

  /** Offer a server name dialog --- returns null if they cancel
   * @param frame Parent frame
   * @return Server name or null
   */
  public static String showSetServerDialog(javax.swing.JFrame frame) {
    return  javax.swing.JOptionPane.showInputDialog(frame,
    "Enter the name of the server\non which you wish to edit source members.\nThe previous edit session will be closed.",
    "Set System",
    javax.swing.JOptionPane.QUESTION_MESSAGE
    );
  }

  /** Offer a File Description user entry dialog --- returns null if they cancel
   * @param frame Parent frame
   * @return Description or null
   */
  public static String showFileDescriptionDialog(javax.swing.JFrame frame) {
    return  javax.swing.JOptionPane.showInputDialog(frame,
    "Enter the description for the new file member:",
    "New file member will be created",
    javax.swing.JOptionPane.QUESTION_MESSAGE
    );
  }

  /** Offer a search string dialog --- returns null if they cancel
   * @param frame Parent frame
   * @return Search spec or null
   */
  public static String showSearchDialog(javax.swing.JFrame frame) {
    return  javax.swing.JOptionPane.showInputDialog(frame,
    "Enter the the string for which to search:",
    "Search for String",
    javax.swing.JOptionPane.QUESTION_MESSAGE
    );
  }

  /** Ask if user wants to save discard or cancel close.
   * @param sys System to use
   * @param ifspath Path of file member
   * @param frame Parent frame
   * @return javax.swing.JOptionPane mask representing button press
   */
  public static int showCloseCancelFileMemberDialog (com.ibm.as400.access.AS400 sys, String ifspath, javax.swing.JFrame frame) {
    return showCloseFileMemberDialog (sys, ifspath, frame, javax.swing.JOptionPane.YES_NO_CANCEL_OPTION);
  }

  /** Ask if user wants to save discard -- can't cancel close.
   * @param sys System to use
   * @param ifspath Path of file member
   * @param frame Parent frame
   * @return javax.swing.JOptionPane mask representing button press
   */
  public static int showCloseFileMemberDialog (com.ibm.as400.access.AS400 sys, String ifspath, javax.swing.JFrame frame) {
    return showCloseFileMemberDialog (sys, ifspath, frame, javax.swing.JOptionPane.YES_NO_OPTION);
  }

  /** Ask if user wants to save discard or also cancel close if option type passed in.
   * @param sys System to use
   * @param ifspath Path of file member
   * @param frame Parent frame
   * @param option avax.swing.JOptionPane mask representing number of buttons to present
   * @return javax.swing.JOptionPane mask representing button press
   */
  public static int showCloseFileMemberDialog(com.ibm.as400.access.AS400 sys, String ifspath, javax.swing.JFrame frame, int option) {
    int result = javax.swing.JOptionPane.CANCEL_OPTION;

    if (null != sys) {
      result =
      javax.swing.JOptionPane.showConfirmDialog
      (frame,
      "File " + sys.getSystemName() + ":" + ifspath + " is changed, save it before closing?",
      "File Changed - Save?",
      option,
      javax.swing.JOptionPane.QUESTION_MESSAGE
      );
    }
    return result;
  }

  /** Ask if user wants to overwrite local file.
   * @paramt path The local path of the local file user has asked to save local.
   * @param frame Parent frame
   * @param option javax.swing.JOptionPane mask representing number of buttons to present
   * @return javax.swing.JOptionPane mask representing button press
   */
  public static int showOverwriteExtantLocalFileDialog(String path, javax.swing.JFrame frame) {
    int result =
    javax.swing.JOptionPane.showConfirmDialog
    (frame,
    "Local file " + path + " already exists, overwrite?",
    "Saving Local File - Overwrite Extant File?",
    javax.swing.JOptionPane.OK_CANCEL_OPTION,
    javax.swing.JOptionPane.QUESTION_MESSAGE
    );
    return result;
  }

  /** Show the about dialog
   * @param icon Icon to use
   */
  public static void showAboutSoftWoehrDialog(javax.swing.ImageIcon icon) {
    javax.swing.JOptionPane.showMessageDialog(null,
    "Get the latest SoftWoehr from http://www.softwoehr.com\nSoftWoehr MyEditUtil and QCOpNav are Copyright *C* 2000 by Jack J. Woehr.\nThey are distributed with source as IBM Public License software.\nCopyright *C* 2000 IBM and others.",
    "MEU and QCOpNav coded by Jack. J. Woehr",
    javax.swing.JOptionPane.INFORMATION_MESSAGE, icon);
  }

  /** Show the help dialog
   * @param icon Icon to use
   */
  public static void showHelpDialog(javax.swing.ImageIcon icon) {
    javax.swing.JOptionPane.showMessageDialog(null,
    "Quel dommage, but currently the only help is the flyover text.", "(Currently No) Help for SoftWoehr MEU and OpNav (Sigh)",
    javax.swing.JOptionPane.INFORMATION_MESSAGE, icon);
  }
}
