/*
 * FormatTool.java
 *
 * Created on July 28, 2000, 4:41 PM
 * This is free open source software distributed under the IBM Public License found
 * on the World Wide Web at http://oss.software.ibm.com/developerworks/opensource/license10.html
 * Copyright *C* 2000, Jack J. Woehr, PO Box 51, Golden, CO 80402-0051 USA jax@well.com
 * Copyright *C* 2000, International Business Machines Corporation and others. All Rights Reserved.
 */

package com.SoftWoehr.JTOpenContrib.MEU.formats;

import java.io.*;
import com.ibm.as400.access.*;

/** Implements the example in the JTOpen docs of autogenerating java source
 * for a particular record format. Just an interesting example tool.
 * @author jax
 * @version 1.0
 */
public class FormatTool extends Object {

  /** Creates new FormatTool */
  public FormatTool() {
  }

  /** Retrieve a record format and generate Java source for that format.
   * @param argv The command line - Usage: FormatTool systemName recordFilepath packageName javaFilePath
   */
  public static void main(java.lang.String[] argv) {
    // Create an AS400 object, the file exists on this
    // AS/400.

    if (4 != argv.length) {
      System.out.println("Usage: FormatTool systemName recordFilepath packageName javaFilePath");
      System.exit(1);
    }

    String systemName = argv[0];
    String recordFilePath = argv[1];
    String packageName = argv[2];
    String javaFilePath = argv[3];

    AS400 sys = new AS400(systemName);

    // Create an AS400FileRecordDescription object that represents the file
    AS400FileRecordDescription myFile = new AS400FileRecordDescription(sys, recordFilePath);

    // Create the Java source file in the current working directory.
    // Specify "package com.myCompany.myProduct;" for the
    // package statement in the source since I will ship the class
    // as part of my product.
    try {
      myFile.createRecordFormatSource(javaFilePath, packageName);
    }
    catch (com.ibm.as400.access.AS400Exception e) {
      e.printStackTrace(System.err);
    }
    catch (com.ibm.as400.access.AS400SecurityException e) {
      e.printStackTrace(System.err);
    }
    catch (IOException e) {
      e.printStackTrace(System.err);
    }
    catch (InterruptedException e) {
      e.printStackTrace(System.err);
    }

    sys.disconnectAllServices();

    System.exit(0);

    // Assuming that the format name for file MYFILE is FILE1, the
    // file FILE1Format.java will be created in the current working directory.
    // It will overwrite any file by the same name.  The name of the class
    // will be FILE1Format.  The class will extend from RecordFormat.

  }
}